//
// Copyright 2021, Jesús Pérez Lorenzo
//
//use std::collections::HashMap;
use serde::{Serialize,Deserialize,};
use anyhow::{Result};
use std::{fs}; //,io};
use std::fs::OpenOptions;
use std::io::{Write};
use std::path::Path;

// use crate::{
//   config::{Config,WebServer},
//   module::Module,
// 	appinfo::AppInfo,
// };

#[derive(Clone, Debug, Serialize, Deserialize)]
pub struct AppLogs {
  pub id: String,
	pub when: String,
	pub name: String,
	pub source: String,
	pub target: String,
	pub state: String,
	pub msg: String
}

impl Default for AppLogs {
  fn default() -> Self {
    Self {
			id: String::from(""),
			name: String::from(""),
			when: format!("{}", chrono::Utc::now().timestamp()),
			source: String::from(""),
			target: String::from(""),
			state: String::from(""),
			msg: String::from(""),
    }
  }
}
impl AppLogs {
  pub async fn write_applogs_entry(&self, output_path: &str) -> Result<()> {
		let mut out = serde_json::to_string(&self).unwrap_or_else(|e|{
			eprintln!("Error write to applogs: {}",e);
			String::from("")
		});
		if out.is_empty() {
			return Ok(());
		}
		out = format!("{},\n",&out);
		if Path::new(&output_path).exists() {
			let mut file = OpenOptions::new().append(true).open(&output_path)?;
			// out = out.replace("&#x2F;", "/");
			file.write(out.as_bytes())?;
		} else {
			let mut file = OpenOptions::new().write(true).create(true).open(&output_path)?;
			file.write_all(out.as_bytes())?;
		}
		Ok(())
  }
  pub async fn load_applogs_entries(source_path: &str) -> Result<Vec<AppLogs>> {
		let applogs: Vec<AppLogs>;
		if Path::new(&source_path).exists() {	
			let content = fs::read_to_string(&source_path).unwrap_or_else(|e| {
				println!("Failed to read json 'applogs_path' from {}: {}", &source_path,e);
				String::from("")
			});
			if content.is_empty() {
			  return Ok(Vec::new());
			}
			applogs = serde_json::from_str(&content).unwrap_or_else(|e|{
				eprintln!("Error reading applogs {}: {}",&source_path,e);
			  Vec::new()
			});
		} else {
			applogs = Vec::new();
		}
		Ok(applogs)
	}		
  pub fn get_applogs_entries(entries: Vec<AppLogs>, criteria: AppLogs) -> Vec<AppLogs> {
		entries.iter().filter(|entry|
      (!criteria.id.is_empty() && entry.id == criteria.id)
        || (!criteria.name.is_empty() && entry.name == criteria.name)
        || (!criteria.state.is_empty() && entry.state.contains(&criteria.state)) 
        || (!criteria.source.is_empty() && entry.source.contains(&criteria.source)) 
        || (!criteria.target.is_empty() && entry.target.contains(&criteria.target)) 
        || (!criteria.when.is_empty() && entry.when.contains(&criteria.when)) 
        || (!criteria.msg.is_empty() && entry.msg.contains(&criteria.msg))
		).map(|entry|entry.to_owned()).collect()
	}		
}